# the usual optimization code, copied from regular_minimal.py

import linear
from math import sqrt, log

def log2(f):
    return log(f)/log(2)

primeNumbers = (2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31)

def write_lts(primes, etCutoff=0.17,
        nETs=20, highestComplexity=2, worstError=0.002, output=1):
    # make adjustments
    highestComplexity *= len(primes)**2 * 0.125/primes[0]
    etCutoff /= sqrt(len(primes))

    ets = linear.getEqualTemperaments(primes, nETs=nETs, cutoff=etCutoff)
    if output:
        print ' '.join([str(et[0]) for et in ets])

    for lt in linear.getLinearTemperaments(ets, primes, highestComplexity, worstError):
        if output:
            print lt.invariant(),
    if output:
        print "\n"

tubulongFrequencies = [
    float(s*(s*s - 1)) / sqrt((s*s + 1) * 7.2)
    for s in range(3,9)]

tubulongPrimes = map(log2, tubulongFrequencies)

def test(output=0):
    limits = range(2, 11) # indexes primes (or primeNumbers)
    if output:
        print "RMS Optima\n"
    for limit in limits:
        if output:
            print "%2i-limit" % primeNumbers[limit]
        write_lts(linear.primes[:limit+1], output=output)

    if output:
        print "Less good ETs\n"
    for limit in limits:
        if output:
            print "%2i-limit" % primeNumbers[limit]
        write_lts(linear.primes[:limit+1], etCutoff=0.22, worstError=0.005, output=output)

    if output:
        print "Microtemperaments\n"
    for limit in limits:
        if output:
            print "%2i-limit" % primeNumbers[limit]
        write_lts(linear.primes[:limit+1], nETs=30, highestComplexity=5, worstError=5e-4, output=output)

    for f in tubulongFrequencies:
        if output:
            print "%.5f"%f,
    print ''

    if output:
        print "Octave-Equivalent Tubulongs\n"
    for limit in range(2, len(tubulongPrimes)+1):
        write_lts(([1.0]+tubulongPrimes)[:limit+1],  worstError = 0.002, output=output)

    if output:
        print "Straight Tubulongs\n"
    for limit in range(2, len(tubulongPrimes)):
        write_lts(tubulongPrimes[:limit+1], worstError = 0.002, output=output)

if __name__=='__main__':
    import sys
    if len(sys.argv)>1 and sys.argv[1].startswith('pro'):
        import profile, pstats
        profile.run('test()', 'linear.prof')
        pstats.Stats('linear.prof').strip_dirs().sort_stats('cumulative').print_stats(20)
    else:
        test(output=1)
