from __future__ import division
import urllib, math
import regular, parametric, regutils, names, kernel

cents = 1200.0

bra = '<tr><td align="right">&lt;</td><td align="right">%s</td><td align="left">]</td></tr>'
ket = '<tr><td align="right">[</td><td align="right">%s</td><td align="left">&rt;</td></tr>'
label = '<tr><th></th><th>%s</th><th></th></tr>'
etLinkFormat = "<a href=rt.cgi?%s>%s</a>"

listFormat = """<table>
    <tr><th></th><th>complexity</th><th>error (cents)</th><th></th></tr>
    %s
</table>"""
rowFormat = """<tr>
    <td>%s</td>
    <td align="right">%.*f</td>
    <td align="right"><a href="pregular.cgi?%s">%.*f</a></td>
    <td>&nbsp;&nbsp;&nbsp;&nbsp;%s</td>
</tr>"""
linkFormat = '<a href="rt.cgi?%s">%s</a>'

EqualTemperamentFormat = """
<h2>%(name)s</h2>
%(mapping)s
<div>&nbsp;</div>
<div>Tuning Map (cents)</div>
<%(tuningMap)s]
<div>&nbsp;</div>
<table>
    <tr>
        <td>Adjusted Error</td>
        <td align="right">%(error).6f</td><td>cents</td>
    </tr>
    <tr>
        <td>TE Error</td>
        <td align="right">%(topError).6f</td><td>cents/octave</td>
    </tr>
    <tr>
        <td>TE Stretch</td>
        <td align="right">%(stretch).6f</td><td>cents/octave</td>
    </tr>
</table>
%(uvs)s
"""

regularTemperamentFormat = """
<h2>%(name)s</h2>
Equal Temperament Mappings
%(melody)s
<div>&nbsp;</div>
<div>Reduced Mapping</div>
%(mapping)s
<div>&nbsp;</div>
<div>Generator Tunings (cents)</div>
<%(generators)s]
<div>&nbsp;</div>
<div>Step Tunings (cents)</div>
<%(steps)s]
<div>&nbsp;</div>
<div>Tuning Map (cents)</div>
<%(tuningMap)s]
<div>&nbsp;</div>
<table>
    <tr>
        <td>Complexity</td>
        <td>%(complexity).6f</td></tr>
    <tr>
        <td>Adjusted Error</td>
        <td>%(error).6f cents</td>
    </tr>
    <tr>
        <td>TE Error</td>
        <td>%(topError).6f cents/octave</td>
    </tr>
</table>
%(uvs)s
%(scala)s
"""
scalaFormat = """<div>&nbsp;</div>
<div>Scala Files</div>
<table>
    %s
</table>
<div>&nbsp;</div>
"""
scalatitles = '<tr><td></td><td colspan="%i">Number of notes per octave</td>'
scalalink = linkFormat.replace("rt.cgi", "scala.cgi")
scalatoprow = '<tr><td>TE</td><td>%s</td><td></tr>'
scalaoerow = '<tr><td>Pure octaves</td><td>%s</td><td></tr>'

searchCaveat = """<p><font size="-1">
Only one temperament per class is shown.
They should be the best according to the relevant
<a href="/badness.pdf">parametric badness</a> but the
search may not be complete, especially in higher prime limits.
Contorsion is not detected so some results may not be temperaments
in the strict sense (they have
the same error but a multiple of the complexity of a torsion-free
temperament).
Names follow the best consensus I can determine or whatever caprice
seizes me at the time.
No wedgies were harmed in the making of this list.
</font></p>"""

scalatext = """! regular.scl
!
%(nnotes)i note scale from some %(name)s temperament.
! %(octaves)s Fokker block.
! Generated by http://x31eq.com/temper/
%(nnotes)i
!
%(notes)s
"""

def formatRT(plimit, labels, mapping, melody):
    """Hide all this ???Printer rubbish"""
    return str(formatter(plimit, labels, mapping, melody))

def formatter(plimit, labels, mapping, melody):
    """Return the formatter object instead of a string"""
    rank = len(mapping)
    if rank==1:
        return EqualTemperamentPrinter(mapping[0], plimit, labels)
    elif rank==2:
        return Rank2TemperamentPrinter(plimit, labels, melody)
    else:
        return RegularTemperamentPrinter(plimit, labels, mapping, melody)

def formatList(rts, labels, errorstr=None):
    rows = []
    for rt in rts:
        try:
            rt = regular.RegularTemperament(rt.plimit, rt.melody)
        except AttributeError:
            pass # Already the right object
        complexity = rt.complexity()
        error = rt.optimalError() * cents * max(rt.primes)
        warts = regutils.wartsFromPrimeLabels(labels)
        etNames = [regular.EqualTemperament(mapping, rt.primes).name(warts)
                for mapping in rt.melody]
        params = {
                'ets' : '_'.join(etNames),
                'limit' : regutils.formatPrimeLabels(labels, '_'),
                }
        name, ignored = displayName(rt, labels)
        name += "+"*ignored
        if len(labels) - len(etNames) == 1: # only one unison vector
            if ignored != 0 or '&' in name: # no explicitly assigned name
                uvs = kernel.unison_vectors(rt.melody, rt.primes)
                if len(uvs)==1: # it had better be
                    name = simpleVector(uvs[0], labels)

        link = linkFormat % (urllib.urlencode(params), name.encode('utf8'))
        links = [etLink(et, labels, rt.primes) for et in rt.melody]
        ets = " &amp; ".join(links)
        errorSearch = {
                "limit":regutils.formatPrimeLabels(labels, '_'),
                "error":"%.*f" % (setPrecision(error), error),
                }
        rows.append(rowFormat % (link,
            setPrecision(complexity), complexity,
            urllib.urlencode(errorSearch),
            setPrecision(error), error, ets))
    return (listFormat % '\n'.join(rows))

def setPrecision(number, default=3):
    if number == 0.0:
        return default
    optimal = int(2 - math.log10(number)) # Should give 2 figures
    return max(default, optimal)

def scalaFile(rt):
    """File for a default scale and current tuning"""
    # not HTML :-O
    scale = rt.tunedBlock()
    rawname, ext = rt.name()
    name = rawname.encode('iso-8859-1') + '+'*ext
    data = {'name':name, 'nnotes':len(scale)}
    data['notes'] = '\n'.join(['%8.3f'%note for note in scale])
    data['octaves'] = ' & '.join([str(x[0]) for x in rt.melody])
    return scalatext % data

class RegularTemperamentPrinter:
    format = regularTemperamentFormat
    def __init__(self, plimit, labels, mapping, melody):
        lt = regular.RegularTemperament(plimit, mapping)
        lt.optimize()
        rt = regular.RegularTemperament(plimit, melody)
        rt.optimize()
        self.standardInitialization(lt, rt, labels)
        self.mapping = self.formatMappings(lt.melody)
        if regutils.lattice_invariant(rt.melody) != lt.invariant():
            self.melody += "<p>These equal temperaments do not constitute a basis.</p>"

    def standardInitialization(self, lt, rt, labels):
        """Used by Regular... and Rank2..."""
        self.labels = labels
        warts = regutils.wartsFromPrimeLabels(labels)
        name, ignored = displayName(rt, labels)
        if ignored==1:
            name += " one dimension higher"
        elif ignored:
            name += " %i dimensions higher" % ignored
        self.name = name.encode('utf-8')
        self.complexity = rt.complexity()
        self.topError = lt.rmsError()*cents
        self.error = self.topError * max(rt.primes)
        self.melody = self.formatMappings(rt.melody)
        self.generators = self.formatCentsList(lt.basis)
        self.steps = self.formatCentsList(rt.basis)
        self.tuningMap = self.formatCentsList(lt.tuningMap())
        self.uvs = self.formatUnisonVectors(rt.melody, rt.primes, labels)
        self.scala = self.formatScala(rt)

    def formatMappings(self, mappings):
        labels = label % '</th><th>'.join(map(str, self.labels))
        bras = [bra % '</td><td align="right">'.join(map(str, mapping))
                for mapping in mappings]
        bras[0] = bras[0].replace('&lt;', '[&lt;')
        bras[-1] = bras[-1].replace(']', ']&gt;')
        body = '\n'.join(bras)
        return '<table><th>%s\n%s</th></table>'%(labels, body)

    def formatCentsList(self, floats):
        return ', '.join(['%.3f' % (f * cents) for f in floats])

    def formatUnisonVectors(self, melody, plimit, labels):
        if len(melody) == len(labels):
            # Just intonation, or something like it
            # no unison vectors
            return ""
        title = "<div>&nbsp;</div><div>Unison Vectors</div>"
        uvs = kernel.more_uvs(melody, plimit)
        if len(uvs)==1:
            formatted = formatUnisonVector(uvs[0], labels)
            return title.replace("ors", "or") + formatted
        formatted = [formatUnisonVector(uv, labels) for uv in uvs]
        assert formatted
        return "%s<ul><li>%s</li></ul>" % (title, "</li><li>".join(formatted))

    def formatScala(self, rt):
        steps = [x[0] for x in rt.melody]
        rows = [scalatitles%len(steps)]
        toprms = []
        oe = []
        params = {
                'key' : '_'.join(map(str, rt.invariant())),
                'limit' : regutils.formatPrimeLabels(self.labels, '_'),
                }
        oetag = '&tuning=po'

        for i in range(len(steps)):
            newsteps = [steps[i]] + steps[:i] + steps[i+1:]
            params['ets'] = '_'.join(map(str, newsteps))
            name = str(newsteps[0])
            toprms.append(scalalink % (urllib.urlencode(params), name))
            oe.append(scalalink % (urllib.urlencode(params)+oetag, name))
        rows.append(scalatoprow % '</td><td>'.join(toprms))
        rows.append(scalaoerow % '</td><td>'.join(oe))

        return scalaFormat % ('\n'.join(rows))

    def __str__(self):
        return self.format % self.__dict__

class EqualTemperamentPrinter(RegularTemperamentPrinter):
    format = EqualTemperamentFormat
    def __init__(self, mapping, plimit, labels):
        self.labels = labels
        self.mapping = self.formatMappings([mapping])
        w = [x/p/mapping[0] for x, p in zip(mapping, plimit)]
        mw = regutils.mean(w)
        msw = regutils.rms(w)**2
        self.topError = math.sqrt(1 - mw**2/msw)*cents
        self.error = self.topError * max(plimit)
        stretch = mw/msw
        self.stretch = (stretch / plimit[0] - 1) * cents
        tuningMap = [x*stretch/mapping[0] for x in mapping]
        self.tuningMap = self.formatCentsList(tuningMap)
        warts = regutils.wartsFromPrimeLabels(labels)
        self.name = regular.EqualTemperament(mapping, plimit).name(warts)
        self.uvs = self.formatUnisonVectors([mapping], plimit, labels)

class Rank2TemperamentPrinter(RegularTemperamentPrinter):
    def __init__(self, plimit, labels, melody):
        lt = regular.LinearTemperament(melody[0], melody[1], plimit)
        lt.optimizeRMS()
        rt = regular.RegularTemperament(plimit, melody)
        rt.optimize()
        self.standardInitialization(lt, rt, labels)
        self.mapping = self.formatMappings(
                (lt.periodMapping, lt.generatorMapping))

def displayName(rt, labels):
    try:
        return regutils.nameThatMapping(rt.melody, labels)
    except KeyError:
        warts = regutils.wartsFromPrimeLabels(labels)
        return ' & '.join(
            [regular.EqualTemperament(mapping, rt.primes).name(warts)
                for mapping in rt.melody]), 0

def etLink(mapping, labels, plimit=None):
    parameters = {'key' : '_'.join(map(str, mapping)),
            'ets' : mapping[0],
            'limit' : regutils.formatPrimeLabels(labels, '_'),
            }
    if plimit:
        warts = regutils.wartsFromPrimeLabels(labels)
        name = regular.EqualTemperament(mapping, plimit).name(warts)
    else:
        name = str(mapping[0])
    return etLinkFormat % (urllib.urlencode(parameters), name)

def formatUnisonVector(vector, labels):
    ket = "[%s>" % ",&nbsp;".join(map(str, vector[:len(labels)]))
    params = urllib.urlencode({
            'limit' : regutils.formatPrimeLabels(labels, '_'),
            'uvs' : simpleVector(vector, labels),
            })
    try:
        n, d = regutils.getRatio(vector, labels)
        factor = regular.hcf((n, d))
        n //= factor
        d //= factor
        return '%s (<a href="uv.cgi?%s">%i:%i</a>)' % (ket, params, n, d)
    except ValueError:
        return '<a href="uv.cgi?%s">%s</a>' % (params, ket)

def simpleVector(vector, labels):
    """Simple formatting of a unison vector for links"""
    ket = "[%s>" % ",".join(map(str, vector[:len(labels)]))
    for label in labels:
        if type(label) != int:
            return ket
    ratio = "%i:%i" % regutils.getRatio(vector, labels)
    if len(ratio) < len(ket):
        return ratio
    return ket
