# Works like regular_oe but uses optimal RMS error and new complexity
from __future__ import generators

import regular_oe as oe

# some functions copied from regular_oe.py
# so they work with our new class

def getLinearTemperaments(ets,
        highestComplexity,
        highestError,
        nResults=100):
    return oe.getLinearTemperaments(ets, highestComplexity, highestError,
            nResults=nResults, temperamentFactory=LinearTemperament)

def Temperament(primes, m, n=None):
    et1 = oe.et.BestET(m, primes)
    if n is None:
        return et1
    et2 = oe.et.BestET(n, primes)
    return LinearTemperament(et1, et2)

temperament = Temperament

# class that uses the right error and complexity
class LinearTemperament(oe.LinearTemperament):

    def optimalError(self):
        """Uses the true optimal scale stretch"""
        # uses the simple calculation based on the zero mean deviation stretch
        shrink1 = self.ets[0].meanerr + 1
        shrink2 = self.ets[1].meanerr + 1
        zmd_cov12 = self.cov12/shrink1/shrink2
        zmd_var1 = self.var1/shrink1**2
        zmd_var2 = self.var2/shrink2**2
        top = zmd_var1*zmd_var2 - zmd_cov12**2
        bottom = top + zmd_var1 + zmd_var2 - zmd_cov12*2
        return oe.sqrt(top/bottom)

    def complexity(self):
        """New complexity"""
        # The size of the weighted mapping matrix
        # calculated using the error matrix
        var1, var2 = self.var1, self.var2
        mean1, mean2 = self.ets[0].meanerr, self.ets[1].meanerr
        meanprod = self.cov12 + mean1*mean2 + mean1 + mean2
        meansq1 = var1 + mean1*mean1 + 2*mean1
        meansq2 = var2 + mean2*mean2 + 2*mean2
        return self.etprod*oe.sqrt(
                meansq1*meansq2 - meanprod*meanprod
                + meansq1 + meansq2 - 2*meanprod)

    # should override the optimize() and error() as well

if __name__=='__main__':
    import sys

    # evil hack to steal the test code from regular_oe.py
    oe.LinearTemperament = LinearTemperament

    if len(sys.argv)>1 and sys.argv[1].startswith('big'):
        oe.bigTest(func=getLinearTemperaments, error=oe.et.rmsError)
    elif len(sys.argv)>1 and sys.argv[1].startswith('huge'):
        oe.bigTest(nETs=1001, func=getLinearTemperaments, error=oe.et.rmsError)
    else:
        oe.test(func=getLinearTemperaments, error=oe.et.rmsError)

