; Alters regular.scm to use STD complexity
; (and, by implication, STD error)
; instead of scalar complexity and TOP-RMS error.

(load "regular.scm")

; This function needs to be duplicated
; so that it returns ETs limited by the standard deviation error
(define (limited-mappings n-notes max-std basis)
    (let (
          (limit (* max-std max-std (length basis)))
          (step-size (/ (car basis) n-notes)))
      (let more-mappings (
            (seed-mapping '())
            (next-guess n-notes)
            (tot 0.0) (tot2 0.0)
            (more-primes basis))
        (let* ((mapping (append seed-mapping (list next-guess)))
               (weighted-size (/ (* next-guess step-size) (car more-primes)))
               ; the next few lines determine the type of error
               (tot (+ tot weighted-size))
               (tot2 (+ tot2 (* weighted-size weighted-size)))
               (error (- tot2 (/ (* tot tot) (length mapping)))))
            (cond
                ((> error limit) '()) ; too bad
                ((= (length mapping) (length basis))
                    ; end of the road
                    (if (= (hcf mapping) 1) (list mapping) '()))
                (else
                     (let ((best-guess (nint
                            (/ (cadr more-primes) step-size))))
                        (define (try-all guess op)
                            (let ((results (
                                  more-mappings
                                      mapping (op guess 1) tot tot2
                                      (cdr more-primes))))
                              (if (> (length results) 0)
                                  (append results (try-all (op guess 1) op))
                                  results)))
                        (append (more-mappings
                                       mapping best-guess tot tot2
                                       (cdr more-primes))
                                (try-all best-guess +)
                                (try-all best-guess -)))))))))

(define (lt.complexity lt)
    (let ((var1 (et.var (lt.et1 lt)))
          (var2 (et.var (lt.et2 lt))))
        (* (lt.etprod lt) (sqrt (- (+ var1 var2) (* 2 (lt.cov12 lt)))))))

; Equation 124 of primerr.pdf
(define (lt.tuning lt)
    (let ((var1 (et.var (lt.et1 lt)))
          (var2 (et.var (lt.et2 lt)))
          (cov12 (lt.cov12 lt)))
        (/ (- var2 cov12) (+ var1 var2 (* -2 cov12)))))

; generators in cents with optimal tuning
(define (lt.g1 lt)
    (* 1200 (/ (lt.tuning lt) (car (et.mapping (lt.et1 lt))))))
(define (lt.g2 lt)
    (* 1200 (/ (- 1 (lt.tuning lt)) (car (et.mapping (lt.et2 lt))))))

; approximation of TOP-RMS stretch; not even the true ZMD stretch
(define (lt.zmd-stretch lt)
    (- 1 (* (lt.tuning lt) (et.mean (lt.et1 lt)))
    (* (- 1 (lt.tuning lt)) (et.mean (lt.et2 lt)))))

; ad hoc approximations of the TOP-RMS generators
(define (lt.g1-zmd lt)
    (* (lt.g1 lt) (lt.zmd-stretch lt)))
(define (lt.g2-zmd lt)
    (* (lt.g2 lt) (lt.zmd-stretch lt)))

