%module temper_c
%{
#include "temper.h"
%}

int_array* int_array_new(int size);
void int_array_set(int_array* this, int index, long value);
long int_array_get(int_array* this, int index);
int int_array_len(int_array* this);
void int_array_free(int_array* this);

float_array* float_array_new(int size);
void float_array_set(float_array* this, int index, double value);
double float_array_get(float_array* this, int index);
int float_array_len(float_array* this);
void float_array_free(float_array* this);

generic_list* generic_list_new(void);
generic_list* generic_list_add(generic_list* old, void* item);
void* generic_list_head(generic_list* this);
generic_list* generic_list_tail(generic_list* this);
void generic_list_free(generic_list* this);

double magnitude(float_array* primes, int_array* interval);

equal_temperament* equal_temperament_list_head(generic_list *this);
equal_temperament* equal_temperament_new(int_array* mapping);
int_array* equal_temperament_mapping(equal_temperament* this);
float_array* equal_temperament_errors(
        equal_temperament* this,
        float_array* primes);
double equal_temperament_resolve(
        equal_temperament* this,
        int_array* interval);
double equal_temperament_worst_error(
        equal_temperament* this,
        generic_list* intervals,
        float_array* primes);
void equal_temperament_free(equal_temperament* this);

generic_list* alternative_mappings(
        equal_temperament* et,
        float_array* primes,
        double cutoff);
equal_temperament* prime_et(long n_notes, float_array* primes);
equal_temperament* best_et(
        long n_notes,
        generic_list* consonances,
        float_array* primes);
generic_list* get_limited_ets(
        generic_list* consonances,
        float_array* primes,
        int n_ets,
        double cutoff);

linear_mapping* linear_mapping_list_head(generic_list *this);
linear_mapping* linear_mapping_new(int_array* map1, int_array* map2);
void linear_mapping_free(linear_mapping* this);

int_array* linear_mapping_et1(linear_mapping* this);
int_array* linear_mapping_et2(linear_mapping* this);
int_array* linear_mapping_period_mapping(linear_mapping* this);
int_array* linear_mapping_generator_mapping(linear_mapping* this);

int_array* linear_mapping_invariant(linear_mapping* this);
long linear_mapping_width(linear_mapping* this, int_array* interval);
int_array* linear_mapping_widest(linear_mapping* this, generic_list* intervals);
long linear_mapping_max_width(linear_mapping* this, generic_list* intervals);

linear_temperament* linear_temperament_new(
        linear_mapping* mapping, double generator_size);
void linear_temperament_free(linear_temperament* this);

linear_mapping* linear_temperament_as_linear_mapping(
        linear_temperament* this);

float_array* linear_temperament_errors(
        linear_temperament* this,
        float_array* primes);
double linear_temperament_resolve(
        linear_temperament* this,
        int_array* interval);
void linear_temperament_optimize_rms(
        linear_temperament* this,
        float_array* primes,
        generic_list* intervals);

double worst_error(float_array *errors, generic_list* intervals);
double rms_error(float_array* errors, generic_list* intervals);

generic_list* get_linear_mappings(
        generic_list *ets,
        generic_list *consonances,
        float_array *primes,
        long max_complexity,
        double worst_error,
        int first_result,
        int last_result);
