# test generating rank 2 temperaments from a list of equal temperaments
# taking only one ET at a time

from regular import *

def write_lts(primes, etCutoff=0.17,
        nETs=20, highestComplexity=2, worstError=0.002,
        error=optimalRMSError, output=1):
    # make adjustments
    highestComplexity *= len(primes)**2 * 0.3
    etCutoff /= sqrt(len(primes))

    ets = getEqualTemperaments(primes, nETs, 5, etCutoff, error)
    if output: print ' '.join([str(et[0]) for et in ets])

    for lt in allRank2Temperaments(ets, highestComplexity, worstError):
        if output: print lt.invariant(),
    if output: print "\n"

def test(output=0):
    limits = range(2, 5) # indexes primes (or primeNumbers)
    if output: print "RMS Optima\n"
    for limit in limits:
        if output: print "%2i-limit" % primeNumbers[limit]
        write_lts(primes[:limit+1], output=output)

    if output: print "Less good ETs\n"
    for limit in limits:
        if output: print "%2i-limit" % primeNumbers[limit]
        write_lts(primes[:limit+1], etCutoff=0.22, worstError=0.005, output=output)

    if output: print "Microtemperaments\n"
    for limit in limits:
        if output: print "%2i-limit" % primeNumbers[limit]
        write_lts(primes[:limit+1], nETs=30, highestComplexity=5, worstError=5e-4, output=output)

    #for f in tubulongFrequencies:
    #  print "%.5f"%f,
    #print ''

    #print "Octave-Equivalent Tubulongs\n"
    #for limit in range(2, len(tubulongPrimes)+1):
    #    write_lts(([1.0]+tubulongPrimes)[:limit+1],  worstError = 0.002)

    #print "Straight Tubulongs\n"
    #for limit in range(2, len(tubulongPrimes)):
    #    write_lts(tubulongPrimes[:limit+1], worstError = 0.002)

if __name__=='__main__':
    import sys
    if len(sys.argv)>1 and sys.argv[1].startswith('pro'):
        import profile, pstats
        profile.run('test()', 'rank2.prof')
        pstats.Stats('rank2.prof').strip_dirs().sort_stats('cumulative').print_stats(20)
    else:
        test(output=1)

# we don't need no stinkin' tabs
# vim: tabstop=99
