let write_lts
        n_ets et_cutoff fod use_rms
        highest_complexity worst_error consonances =
    let ets = Temper.get_limited_ets
            consonances n_ets et_cutoff in (
        List.iter
            (function et ->
                Printf.printf "%d " (List.hd et#mapping))
            ets;
        print_newline();
        List.iter
            (function lt ->
                match (List.hd lt#primes_by_melody) with
                    m, n -> Printf.printf "[%d %d] " m n)
            (Temper.get_linear_temperaments
                ets
                consonances
                (fod consonances)
                (List.length consonances#intervals * highest_complexity)
                worst_error
                use_rms);
        print_endline "\n")

and mos_fod intervals lt =
    let width = lt#smallest_containing_mos intervals#intervals
    and error = lt#rms_error intervals
    in
    float_of_int (width*width) *. error
         (* add a fudge factor to give a unique ordering *)
         +. 1e-10 *.
             (let m, n = List.hd lt#primes_by_melody in
             1.618 *. float_of_int m +. float_of_int n)

and gaussian_fod intervals lt =
    let error = lt#rms_error intervals
    and width = lt#max_width intervals#intervals
         * lt#periods_per_octave
    and std_error = 17.0 /. 12e2
    in
    float_of_int(width*width) *. exp((error/.std_error*.3.)**2.)
         (* add a fudge factor to give a unique ordering *)
         +. 1e-10 *.
             (let m, n = List.hd lt#primes_by_melody in
             1.618 *. float_of_int m +. float_of_int n)

and tubulong_frequencies = List.map
    (function s ->
        let squared n = n*n in
        sqrt( (float_of_int(s*(s*s - 1)))**2. /.
                 float_of_int(s*s + 1) )
             /. sqrt(36./.5.))
    [3;4;5;6;7;8]
in
let tubulong_primes = List.map Temper.log2 tubulong_frequencies
in
begin
    List.iter
        (function tag, procedure ->
            begin
                print_endline tag;
                List.iter procedure
                      (List.map Temper.odd_limit [5;7;9;11;13;15;17;19;21])
            end)
        [
            "RMS Optima\n", (write_lts
                20 0.501 Temper.standard_figure_of_demerit true 4 0.3);
            "Minimax Optima\n", (write_lts
                20 0.501 Temper.minimax_figure_of_demerit false 4 0.3);
            "Inconsistent ETs\n", (write_lts
                20 0.6 Temper.standard_figure_of_demerit true 4 0.3);
            "MOS FoD\n", (write_lts
                20 0.501 mos_fod true 4 0.3);
            "Gaussian FoD\n", (write_lts
                25 0.501 gaussian_fod true 6 1.0);
            "Microtemperaments\n", (write_lts
                25 0.501 Temper.standard_figure_of_demerit true 6 (2. /. 12e2));
        ];
    List.iter (Printf.printf "%.5f ") tubulong_frequencies;
    print_newline();
    List.iter
        (function tag, primes ->
        begin
            print_endline tag;
            for dimensions = 2 to (List.length primes) do
                write_lts
                    20 0.501 Temper.standard_figure_of_demerit true 4 0.02
                    (Temper.harmonic_diamond primes dimensions);
            done
        end)
        [
            "Octave-Equivalent Tubulongs\n", tubulong_primes;
            let tubulong_equivalence = List.hd tubulong_primes
            in
            (Printf.sprintf
                "Tubulongs with Equivalence Interval of %.6f\n"
                tubulong_equivalence),
            (List.map
                (( *.) (1. /. tubulong_equivalence))
                (List.tl tubulong_primes))
        ]
end


