#!/usr/bin/python 
# duplicate the output of write_temper.ml

import sys, math
sys.path.append('/home/graham/net/microtonal')
import temper

def figureOfDemeritByMOS(self, consonances=None):
    error = self.lastOptimum = self.getRMSError(consonances)
    width = self.getSmallestContainingMOS(mustBeProper=0)
    m, n = self.primesByMelody[0]
    return width*width*error + 1e-10*(1.618*m+n)

def gaussianFOD(self, consonances=None):
    error = self.lastOptimum = self.getRMSError(consonances)
    width = self.lastComplexity = self.getComplexity(consonances)
    m, n = self.primesByMelody[0]
    return width**2 * math.exp((error/self.stdError*3)**2) + 1e-10*(1.618*m+n)

def rmsFOD(self, consonances=None):
    error = self.getRMSError(consonances)
    width = self.getComplexity(consonances)
    m, n = self.primesByMelody[0]
    return width*width*error + 1e-10*(1.618*m+n)

def write_lts(consonances, useRMS=1, etCutoff=0.501, fod=None,
        nETs=20, highestComplexity=4, worstError=0.3):
    ets = temper.getLimitedETs(consonances, nETs, etCutoff)
    print ' '.join(map(str, ets)) + ' '

    for lt in temper.getLinearTemperaments(
            ets,
            consonances,
            fod or (temper.defaultFigureOfDemerit, rmsFOD)[useRMS],
            len(consonances)*highestComplexity,
            worstError,
            useRMS):
        print "[%d %d]" % lt.primesByMelody[0],
    print "\n"

oddLimits = map(temper.OddLimit, (5,7,9,11,13,15,17,19,21))

tubulongFrequencies = []
for s in range(3,9):
  tubulongFrequencies.append(
      math.sqrt( float(s*(s*s - 1))**2 / (s*s + 1))
      / math.sqrt(36.0/5))

primes = map(temper.log2, tubulongFrequencies)
equivalence = temper.log2(tubulongFrequencies[0])
equiv_primes = []
for f in tubulongFrequencies[1:]:
    equiv_primes.append(temper.log2(f)/equivalence)

if __name__=='__main__':
    print "RMS Optima\n"
    for limit in oddLimits:
      write_lts(limit)

    print "Minimax Optima\n"
    for limit in oddLimits:
      write_lts(limit, useRMS=0)

    print "Inconsistent ETs\n"
    for limit in oddLimits:
      write_lts(limit, etCutoff=0.6)

    print "MOS FoD\n"
    for limit in oddLimits:
      write_lts(limit, fod=figureOfDemeritByMOS)

    print "Gaussian FoD\n"
    for limit in oddLimits:
      write_lts(limit, fod=gaussianFOD, nETs=25, highestComplexity=6, worstError=1.0)

    print "Microtemperaments\n"
    for limit in oddLimits:
      write_lts(limit, nETs=25, highestComplexity=6, worstError=2/12e2)

    for f in tubulongFrequencies:
      print "%.5f"%f,
    print ''

    print "Octave-Equivalent Tubulongs\n"
    for dimensions in range(2, len(primes)+1):
        limit = temper.PrimeDiamond(dimensions)
        limit.primes = primes[:dimensions]
        write_lts(limit, worstError = 0.02)

    print "Tubulongs with Equivalence Interval of %.6f\n"%equivalence
    for dimensions in range(2, len(equiv_primes)+1):
        limit = temper.PrimeDiamond(dimensions)
        limit.primes = equiv_primes[:dimensions]
        write_lts(limit, worstError = 0.02)

