#!/usr/bin/python 
# duplicate the output of write_temper_rms.py
# but use the C library

import sys, math
import temper, temper_wrap

def write_lts(consonances, etCutoff=0.5,
        nETs=20, highestComplexity=4, worstError=0.3):
    ets = temper_wrap.get_limited_ets(
            consonances, consonances.primes, nETs, etCutoff)
    ets.reverse()
    for et in ets:
        print et.mapping[0],
    print

    for lt in temper_wrap.get_linear_mappings(
            ets,
            map(temper.reduceWithinTritone, consonances),
            consonances.primes,
            len(consonances)*highestComplexity,
            worstError,
            0,99):
        print lt.invariant(),
    print "\n"

oddLimits = map(temper.OddLimit, (5,7,9,11,13,15,17,19,21))

tubulongFrequencies = []
for s in range(3,9):
  tubulongFrequencies.append(
      math.sqrt( float(s*(s*s - 1))**2 / (s*s + 1))
      / math.sqrt(36.0/5))

primes = map(temper.log2, tubulongFrequencies)
equivalence = temper.log2(tubulongFrequencies[0])
equiv_primes = []
for f in tubulongFrequencies[1:]:
    equiv_primes.append(temper.log2(f)/equivalence)

def test():
    print "RMS Optima\n"
    for limit in oddLimits:
      write_lts(limit)

    print "Inconsistent ETs\n"
    for limit in oddLimits:
      write_lts(limit, etCutoff=0.6)

    print "Microtemperaments\n"
    for limit in oddLimits:
      write_lts(limit, nETs=25, highestComplexity=6, worstError=2/12e2)

    for f in tubulongFrequencies:
      print "%.5f"%f,
    print ''

    print "Octave-Equivalent Tubulongs\n"
    for dimensions in range(2, len(primes)+1):
        limit = temper.PrimeDiamond(dimensions)
        limit.primes = primes[:dimensions]
        write_lts(limit, etCutoff=0.501, worstError = 0.02)
        # different et cutoff to get consistent results for exactly 0.5

    print "Tubulongs with Equivalence Interval of %.6f\n"%equivalence
    for dimensions in range(2, len(equiv_primes)+1):
        limit = temper.PrimeDiamond(dimensions)
        limit.primes = equiv_primes[:dimensions]
        write_lts(limit, worstError = 0.02)

if __name__=='__main__':
    test()
